==> planner-3.9.90-sources.jar: com/xebialabs/deployit/deployment/planner/StepPlan.java <==
package com.xebialabs.deployit.deployment.planner;

import java.io.PrintWriter;
import java.io.Serializable;
import java.io.Writer;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import com.google.common.base.Function;
import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.google.common.collect.Multimap;

import com.xebialabs.deployit.engine.spi.execution.ExecutionStateListener;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.Operation;
import com.xebialabs.deployit.plugin.api.flow.Step;

import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Lists.transform;
import static com.google.common.collect.Sets.newHashSet;
import static java.util.Collections.sort;

public class StepPlan extends Plan {

    public static final Function<StepWithDelta,Step> stepsWithDeltaToSteps = new Function<StepWithDelta, Step>() {
        public Step apply(StepWithDelta input) {
            return input.step;
        }
    };

    private ListMultimap<Integer, StepWithDelta> stepsMap = ArrayListMultimap.create();

    private List<Checkpoint> checkpoints = newArrayList();

    private Delta deltaUnderPlanning;

    public StepPlan(final List<ExecutionStateListener> listeners) {
        super(listeners);
    }

    public void addStep(Step step) {
        stepsMap.put(step.getOrder(), new StepPlan.StepWithDelta(step, deltaUnderPlanning));
    }

    public void addSteps(Collection<Step> steps) {
        for (Step step : steps) {
            addStep(step);
        }
    }

    public List<StepWithDelta> getStepsWithDelta() {
        List<Integer> orderedKeys = newArrayList(stepsMap.keySet());
        sort(orderedKeys);

        List<StepWithDelta> steps = newArrayList();
        for (Integer key : orderedKeys) {
            steps.addAll(stepsMap.get(key));
        }

        return steps;
    }

    public List<Step> getSteps() {
        return transform(getStepsWithDelta(), stepsWithDeltaToSteps);
    }

    public List<Checkpoint> getCheckpoints() {
        return checkpoints;
    }

    @Override
    public Set<Checkpoint> findCheckpoints() {
        return newHashSet(getCheckpoints());
    }

    @Override
    public Writer writePlan(Writer writer) {
        PrintWriter out = new PrintWriter(writer);

        Multimap<Step, Checkpoint> checkpointsByStep = ArrayListMultimap.create();
        for (Checkpoint checkpoint : checkpoints) {
            checkpointsByStep.put(checkpoint.getStep(), checkpoint);
        }

        int index = 0;
        for (StepWithDelta swo : getStepsWithDelta()) {
            index++;
            out.format("%2s. (%3s) - %s  (%s)\n", index, swo.step.getOrder(), swo.step.getDescription(), swo.getClass().getSimpleName());
            if (checkpointsByStep.containsKey(swo.step)) {
                for (Checkpoint checkpoint : checkpointsByStep.get(swo.step)) {
                    out.format("            *checkpoint*  %s\n", checkpoint);
                }
            }
        }

        if (index == 0) {
            out.format("<empty plan>");
        }
        return writer;
    }

    public void setDeltaUnderPlanning(final Delta delta) {
        this.deltaUnderPlanning = delta;
    }

    public static class StepWithDelta {
        private Set<Delta> deltas = newHashSet();
        private Step step;

        public StepWithDelta(Step step, Delta delta) {
            if (delta != null) {
                this.deltas.add(delta);
            }
            this.step = step;
        }

        public StepWithDelta(Step step, Set<Delta> deltas) {
            this.deltas.addAll(deltas);
            this.step = step;
        }

        public Set<Delta> getDeltas() {
            return deltas;
        }

        public Step getStep() {
            return step;
        }
    }

    // TODO collapse this into the StepWithDelta class
    public static class Checkpoint implements Serializable {
        private Delta delta;
        private Operation operation;
        private Step step;

        public Checkpoint(Delta delta, Step step) {
            this(delta, step, delta.getOperation());
        }

        public Checkpoint(Delta delta, Step step, Operation operation) {
            this.delta = delta;
            this.step = step;
            this.operation = operation;
        }

        public Delta getDelta() {
            return delta;
        }

        public Step getStep() {
            return step;
        }

        public Operation getOperation() {
            return operation;
        }

        @Override
        public String toString() {
            switch (operation) {
                case CREATE:
                    return String.format("%s %s", operation, delta.getDeployed());
                case MODIFY:
                    return String.format("%s from %s to %s", operation, delta.getPrevious(), delta.getDeployed());
                case DESTROY:
                    return String.format("%s %s", operation, delta.getPrevious());
                default:
                    return operation.toString();
            }
        }
    }
}

==> planner-4.0.1-sources.jar: com/xebialabs/deployit/deployment/planner/StepPlan.java <==
package com.xebialabs.deployit.deployment.planner;

import java.io.PrintWriter;
import java.io.Serializable;
import java.io.Writer;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import com.google.common.base.Function;
import com.google.common.base.Strings;
import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.google.common.collect.Multimap;

import com.xebialabs.deployit.engine.spi.execution.ExecutionStateListener;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.Operation;
import com.xebialabs.deployit.plugin.api.flow.Step;

import static com.google.common.collect.Lists.newArrayList;
import static com.google.common.collect.Lists.transform;
import static com.google.common.collect.Sets.newHashSet;
import static java.util.Collections.sort;

public class StepPlan extends Plan {

    public static final Function<StepWithDelta,Step> stepsWithDeltaToSteps = new Function<StepWithDelta, Step>() {
        public Step apply(StepWithDelta input) {
            return input.step;
        }
    };

    private ListMultimap<Integer, StepWithDelta> stepsMap = ArrayListMultimap.create();

    private List<Checkpoint> checkpoints = newArrayList();

    private Delta deltaUnderPlanning;

    public StepPlan(String description, final List<ExecutionStateListener> listeners) {
        super(description, listeners);
    }

    public StepPlan(String description, Collection<Step> steps, final List<ExecutionStateListener> listeners) {
        super(description, listeners);
        addSteps(steps);
    }

    public StepPlan(String description, Collection<StepWithDelta> steps, List<Checkpoint> checkpoints, final List<ExecutionStateListener> listeners) {
        super(description, listeners);
        this.checkpoints = checkpoints;

        for (StepWithDelta step : steps) {
            stepsMap.put(step.getStep().getOrder(), step);
        }
    }

    public void addStep(Step step) {
        stepsMap.put(step.getOrder(), new StepPlan.StepWithDelta(step, deltaUnderPlanning));
    }

    public void addSteps(Collection<Step> steps) {
        for (Step step : steps) {
            addStep(step);
        }
    }

    public List<StepWithDelta> getStepsWithDelta() {
        List<Integer> orderedKeys = newArrayList(stepsMap.keySet());
        sort(orderedKeys);

        List<StepWithDelta> steps = newArrayList();
        for (Integer key : orderedKeys) {
            steps.addAll(stepsMap.get(key));
        }

        return steps;
    }

    public List<Step> getSteps() {
        return transform(getStepsWithDelta(), stepsWithDeltaToSteps);
    }

    public List<Checkpoint> getCheckpoints() {
        return checkpoints;
    }

    @Override
    public Set<Checkpoint> findCheckpoints() {
        return newHashSet(getCheckpoints());
    }

    @Override
    public Writer writePlan(Writer writer) {
        PrintWriter out = new PrintWriter(writer);

        out.println(" * " + this.getDescription());
        out.println("   " + Strings.repeat("-", this.getDescription().length()));

        Multimap<Step, Checkpoint> checkpointsByStep = ArrayListMultimap.create();
        for (Checkpoint checkpoint : checkpoints) {
            checkpointsByStep.put(checkpoint.getStep(), checkpoint);
        }

        int index = 0;
        for (StepWithDelta swo : getStepsWithDelta()) {
            index++;
            out.format("%2s. (%3s) - %s  (%s)\n", index, swo.step.getOrder(), swo.step.getDescription(), swo.getClass().getSimpleName());
            if (checkpointsByStep.containsKey(swo.step)) {
                for (Checkpoint checkpoint : checkpointsByStep.get(swo.step)) {
                    out.format("            *checkpoint*  %s\n", checkpoint);
                }
            }
        }

        if (index == 0) {
            out.format("<empty plan>");
        }
        return writer;
    }

    public void setDeltaUnderPlanning(final Delta delta) {
        this.deltaUnderPlanning = delta;
    }

    public static class StepWithDelta {
        private Set<Delta> deltas = newHashSet();
        private Step step;

        public StepWithDelta(Step step, Delta delta) {
            if (delta != null) {
                this.deltas.add(delta);
            }
            this.step = step;
        }

        public StepWithDelta(Step step, Set<Delta> deltas) {
            this.deltas.addAll(deltas);
            this.step = step;
        }

        public Set<Delta> getDeltas() {
            return deltas;
        }

        public Step getStep() {
            return step;
        }
    }

    // TODO collapse this into the StepWithDelta class
    public static class Checkpoint implements Serializable {
        private Delta delta;
        private Operation operation;
        private Step step;

        public Checkpoint(Delta delta, Step step) {
            this(delta, step, delta.getOperation());
        }

        public Checkpoint(Delta delta, Step step, Operation operation) {
            this.delta = delta;
            this.step = step;
            this.operation = operation;
        }

        public Delta getDelta() {
            return delta;
        }

        public Step getStep() {
            return step;
        }

        public Operation getOperation() {
            return operation;
        }

        @Override
        public String toString() {
            switch (operation) {
                case CREATE:
                    return String.format("%s %s", operation, delta.getDeployed());
                case MODIFY:
                    return String.format("%s from %s to %s", operation, delta.getPrevious(), delta.getDeployed());
                case DESTROY:
                    return String.format("%s %s", operation, delta.getPrevious());
                default:
                    return operation.toString();
            }
        }
    }
}
